/* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */

/**
 * Create an ISO-9660 data volume with Rock Ridge and Joliet extensions.
 * Usage is easy:
 *  - Create a new volume.
 *  - Add files and directories.
 *  - Write the volume to a file or create a burn source for use with Libburn.
 */

#ifndef __LIBISOFS
#define __LIBISOFS

/**
 * Data volume.
 * @see volume.h for details.
 */
struct iso_volume;

/**
 * Directory on a volume.
 * @see tree.h for details.
 */
struct iso_tree_dir;

/**
 * File on a volume.
 * @see tree.h for details.
 */
struct iso_tree_file;

/**
 * Possible versions of a file or directory name or identifier.
 */
enum iso_name_version {
	ISO_NAME_FULL,		/**< Full version: either part of the path or user input. */
	ISO_NAME_ISO,		/**< Current ISO level identifier. */
	ISO_NAME_ISO_L1,	/**< ISO level 1 identifier. */
	ISO_NAME_ISO_L2,	/**< ISO level 2 identifier. */
	ISO_NAME_ROCKRIDGE,	/**< Rock Ridge file or directory name. */
	ISO_NAME_JOLIET		/**< Joliet identifier. */
};

/**
 * Create a new volume.
 * The parameters can be set to NULL if you wish to set them later.
 */
struct iso_volume *iso_volume_new(const char *volume_id, const char *publisher_id, const char *data_preparer_id);

/**
 * Free a volume.
 */
void iso_volume_free(struct iso_volume *volume);

/**
 * Get the root directory for a volume.
 */
struct iso_tree_dir *iso_volume_get_root(const struct iso_volume *volume);

/**
 * Fill in the volume identifier for a volume.
 */
void iso_volume_set_volume_id(struct iso_volume *volume, const char *volume_id);

/**
 * Fill in the publisher for a volume.
 */
void iso_volume_set_publisher_id(struct iso_volume *volume, const char *publisher_id);

/**
 * Fill in the data preparer for a volume.
 */
void iso_volume_set_data_preparer_id(struct iso_volume *volume, const char *data_preparer_id);

/**
 * Get the current ISO level for a volume.
 */
int iso_volume_get_iso_level(const struct iso_volume *volume);

/**
 * Set the current ISO level for a volume.
 * ISO level must be 1 or 2.
 */
void iso_volume_set_iso_level(struct iso_volume *volume, int level);

/**
 * See if Rock Ridge (POSIX) is enabled for a volume.
 */
int iso_volume_get_rockridge(const struct iso_volume *volume);

/**
 * Enable or disable Rock Ridge (POSIX) for a volume.
 */
void iso_volume_set_rockridge(struct iso_volume *volume, int rockridge);

/**
 * See if Joliet (Unicode) is enabled for a volume.
 */
int iso_volume_get_joliet(const struct iso_volume *volume);

/**
 * Enable or disable Joliet (Unicode) for a volume.
 */
void iso_volume_set_joliet(struct iso_volume *volume, int joliet);

/**
 * Add a file to a directory.
 * @return a pointer to the newly created file.
 */
struct iso_tree_file *iso_tree_add_file(struct iso_tree_dir *parent, const char *path);

/**
 * Add a copy from an existing directory to the tree.
 * Warning: this only adds the directory itself, no files or subdirectories.
 * @return a pointer to the newly created directory.
 */
struct iso_tree_dir *iso_tree_add_dir(struct iso_tree_dir *parent, const char *path);

/**
 * Recursively add an existing directory to the tree.
 * Warning: when using this, you'll lose pointers to files or subdirectories.
 * If you want to have pointers to all files and directories, use iso_tree_add_file and iso_tree_add_dir.
 * @return a pointer to the newly created directory.
 */
struct iso_tree_dir *iso_tree_radd_dir(struct iso_tree_dir *parent, const char *path);

/**
 * Creates a new directory on the volume.
 * @return a pointer to the newly created directory.
 */
struct iso_tree_dir *iso_tree_add_new_dir(struct iso_tree_dir *parent, const char *name);

/**
 * Get the name of a file.
 */
const char *iso_tree_file_get_name(const struct iso_tree_file *file, enum iso_name_version ver);

/**
 * Set the name of a file.
 * The name you input here will be the full name and will be used to derive all other names.
 */
void iso_tree_file_set_name(struct iso_tree_file *file, const char *name);

/**
 * Get the name of a directory.
 */
const char *iso_tree_dir_get_name(const struct iso_tree_dir *dir, enum iso_name_version ver);

/**
 * Set the name of a file.
 * The name you input here will be the full name and will be used to derive all other names.
 */
void iso_tree_dir_set_name(struct iso_tree_dir *dir, const char *name);

/**
 * Recursively print a directory to stdout.
 * @param spaces the initial number of spaces on the left. Set to 0 if you supply a root directory.
 */
void iso_tree_print(const struct iso_tree_dir *root, int spaces);

#endif /* __LIBISOFS */
