/* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */

#include <dirent.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <time.h>

#include "libisofs.h"
#include "tree.h"
#include "util.h"
#include "volume.h"

struct iso_tree_dir *iso_tree_new_root(const struct iso_volume *volume)
{
	struct iso_tree_dir *dir;

	dir = calloc(1, sizeof(struct iso_tree_dir));
	dir->volume = volume;

	return dir;
}

void iso_tree_free(struct iso_tree_dir *root)
{
	int i;

	/* Free names. */
	free(root->name.full);
	free(root->name.iso1);
	free(root->name.iso2);
	free(root->name.rockridge);
	free(root->name.joliet);

	/* Free the children. */
	for (i = 0; i < root->nchildren; i++)
		iso_tree_free(root->children[i]);
	free(root->children);

	/* Free all files. */
	for (i = 0; i < root->nfiles; i++) {
		struct iso_tree_file *file = root->files[i];
		free(file->path);
		free(file->name.full);
		free(file->name.iso1);
		free(file->name.iso2);
		free(file->name.rockridge);
		free(file->name.joliet);
		free(file);
	}
	free(root->files);

	/* Free ourself. */
	free(root);
}

struct iso_tree_file *iso_tree_add_file(struct iso_tree_dir *parent, const char *path)
{
	struct iso_tree_file *file;
	struct stat st;

	/* Make sure we have a valid parent and path, and the path points to a regular file. */
	if (parent == NULL || path == NULL || path[0] != '/' || stat(path, &st) != 0 || !S_ISREG(st.st_mode))
		return NULL;

	/* Make room for one more file pointer, and add one. */
	parent->nfiles++;
	parent->files = realloc(parent->files, sizeof(struct iso_tree_file *) * parent->nfiles);
	file = parent->files[parent->nfiles - 1] = calloc(1, sizeof(struct iso_tree_file));

	/* Set up path, parent and volume. */
	file->path = strdup(path);
	file->parent = parent;
	file->volume = parent->volume;

	/* Set the file name to the final part of the path. */
	iso_tree_file_set_name(file, strrchr(path, '/') + 1);

	/* Set the POSIX attributes. */
	file->attrib.size = st.st_size;
	file->attrib.mode = st.st_mode;
	file->attrib.user = st.st_uid;
	file->attrib.group = st.st_gid;

	file->attrib.mtime = st.st_mtime;
	file->attrib.atime = st.st_atime;
	file->attrib.ctime = st.st_ctime;

	return file;	
}

struct iso_tree_dir *iso_tree_add_dir(struct iso_tree_dir *parent, const char *path)
{
	struct iso_tree_dir *dir;
	struct stat st;

	/* Make sure we have a valid parent and path, and the path points to a directory. */
	if (parent == NULL || path == NULL || path[0] != '/' || stat(path, &st) != 0 || !S_ISDIR(st.st_mode))
		return NULL;

	parent->nchildren++;
	parent->children = realloc(parent->children, sizeof(struct iso_tree_dir *) * parent->nchildren);
	dir = parent->children[parent->nchildren - 1] = calloc(1, sizeof(struct iso_tree_dir));

	dir->parent = parent;
	dir->volume = parent->volume;

	iso_tree_dir_set_name(dir, strrchr(path, '/') + 1);

	dir->attrib.mode = st.st_mode;
	dir->attrib.user = st.st_uid;
	dir->attrib.group = st.st_gid;

	dir->attrib.mtime = st.st_mtime;
	dir->attrib.atime = st.st_atime;
	dir->attrib.ctime = st.st_ctime;

	return dir;
}

struct iso_tree_dir *iso_tree_radd_dir(struct iso_tree_dir *parent, const char *path)
{
	struct iso_tree_dir *nparent;
	struct stat st;
	DIR *dir;
	struct dirent *ent;

	if (parent == NULL || path == NULL || path[0] != '/' || stat(path, &st) != 0 || !S_ISDIR(st.st_mode))
		return NULL;

	nparent = iso_tree_add_dir(parent, path);

	/* Open the directory for reading and iterate over the directory entries. */
	dir = opendir(path);
	while ((ent = readdir(dir))) {
		char *child;

		/* Skip current and parent directory entries. */
		if (strcmp(ent->d_name, ".") == 0 || strcmp(ent->d_name, "..") == 0)
			continue;

		/* Build the child's full pathname. */
		child = iso_pathname(path, ent->d_name);
		
		/* Skip to the next entry on errors. */
		if (stat(child, &st) != 0)
			continue;
		
		if (S_ISDIR(st.st_mode))
			iso_tree_radd_dir(nparent, child);
		else if (S_ISREG(st.st_mode))
			iso_tree_add_file(nparent, child);
		else
			continue;

		free(child);
	}

	closedir(dir);

	return nparent;
}

struct iso_tree_dir *iso_tree_add_new_dir(struct iso_tree_dir *parent, const char *name)
{
	struct iso_tree_dir *dir;

	if (parent == NULL || name == NULL)
		return NULL;

	parent->nchildren++;
	parent->children = realloc(parent->children, sizeof(struct iso_tree_dir *) * parent->nchildren);
	dir = parent->children[parent->nchildren - 1] = calloc(1, sizeof(struct iso_tree_dir));

	dir->parent = parent;
	dir->volume = parent->volume;

	iso_tree_dir_set_name(dir, name);

	/* Set up sane defaults. */
	dir->attrib.mode = S_IFDIR | S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH;
	dir->attrib.user = parent->attrib.user;
	dir->attrib.group = parent->attrib.group;

	dir->attrib.mtime = time(NULL);
	dir->attrib.atime = time(NULL);
	dir->attrib.ctime = time(NULL);

	return dir;
}

const char *iso_tree_file_get_name(const struct iso_tree_file *file, enum iso_name_version ver)
{
	if (ver == ISO_NAME_FULL)
		return file->name.full;
	else if (ver == ISO_NAME_ISO && file->volume->options.iso_level == 1 || ver == ISO_NAME_ISO_L1)
		return file->name.iso1;
	else if (ver == ISO_NAME_ISO && file->volume->options.iso_level == 2 || ver == ISO_NAME_ISO_L2)
		return file->name.iso2;
	else if (ver == ISO_NAME_ROCKRIDGE)
		return file->name.rockridge;
	else if (ver == ISO_NAME_JOLIET)
		return file->name.joliet;
	else
		return NULL;
}

void iso_tree_file_set_name(struct iso_tree_file *file, const char *name)
{
	file->name.full = strdup(name);
	file->name.iso1 = iso_1_fileid(name);
	file->name.iso2 = iso_2_fileid(name);
	file->name.rockridge = iso_p_filename(name);
	file->name.joliet = iso_j_id(name, &file->name.jsize);
}

const char *iso_tree_dir_get_name(const struct iso_tree_dir *dir, enum iso_name_version ver)
{
	if (ver == ISO_NAME_FULL)
		return dir->name.full;
	else if (ver == ISO_NAME_ISO && dir->volume->options.iso_level == 1 || ver == ISO_NAME_ISO_L1)
		return dir->name.iso1;
	else if (ver == ISO_NAME_ISO && dir->volume->options.iso_level == 2 || ver == ISO_NAME_ISO_L2)
		return dir->name.iso2;
	else if (ver == ISO_NAME_ROCKRIDGE)
		return dir->name.rockridge;
	else if (ver == ISO_NAME_JOLIET)
		return dir->name.joliet;
	else
		return NULL;
}

void iso_tree_dir_set_name(struct iso_tree_dir *dir, const char *name)
{
	dir->name.full = strdup(name);
	/* Level 1 directory is a string of d-characters of maximum size 8. */
	dir->name.iso1 = iso_d_str(name, 8, 0);
	/* Level 2 directory is a string of d-characters of maximum size 31. */
	dir->name.iso2 = iso_d_str(name, 31, 0);
	dir->name.rockridge = iso_p_dirname(name);
	dir->name.joliet = iso_j_id(name, &dir->name.jsize);
}

/* Compares files using their current ISO level name. */
static int iso_file_cmp(const void *v1, const void *v2)
{
	struct iso_tree_file **f1 = (struct iso_tree_file **)v1;
	struct iso_tree_file **f2 = (struct iso_tree_file **)v2;

	return strcmp(iso_tree_file_get_name(*f1, ISO_NAME_ISO), iso_tree_file_get_name(*f2, ISO_NAME_ISO));
}

/* Compares directories using their current ISO level name. */
static int iso_dir_cmp(const void *v1, const void *v2)
{
	struct iso_tree_dir **d1 = (struct iso_tree_dir **)v1;
	struct iso_tree_dir **d2 = (struct iso_tree_dir **)v2;

	return strcmp(iso_tree_dir_get_name(*d1, ISO_NAME_ISO), iso_tree_dir_get_name(*d2, ISO_NAME_ISO));
}

void iso_tree_sort(struct iso_tree_dir *root)
{
	int i;

	qsort(root->files, root->nfiles, sizeof(struct iso_tree_file *), iso_file_cmp);

	qsort(root->children, root->nchildren, sizeof(struct iso_tree_dir *), iso_dir_cmp);

	for (i = 0; i < root->nchildren; i++)
		iso_tree_sort(root->children[i]);
}

void iso_tree_print(const struct iso_tree_dir *root, int spaces)
{
	int i, j;

	for (i = 0; i < spaces; i++)
		printf(" ");

	/* Root directory doesn't have a name. */
	if (root->name.full != NULL)
		printf("%s", iso_tree_dir_get_name(root, ISO_NAME_ISO));
	printf("/\n");

	spaces += 2;

	for (j = 0; j < root->nchildren; j++)
		iso_tree_print(root->children[j], spaces);

	for (j = 0; j < root->nfiles; j++) {
		for (i = 0; i < spaces; i++)
			printf(" ");
		printf("%s\n", iso_tree_file_get_name(root->files[j], ISO_NAME_ISO));
	}
}
