/* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */

/**
 * Extra declarations for use with the iso_tree_dir and iso_tree_file structures.
 */

#ifndef __ISO_TREE
#define __ISO_TREE

#include <sys/types.h>
#include <time.h>

#include "libisofs.h"

/**
 * File or directory names or identifiers.
 */
struct iso_names {
	char *full;		/**< Full version: either part of the path or user input. */
	char *iso1;		/**< ISO level 1 identifier. */
	char *iso2;		/**< ISO level 2 identifier. */
	char *rockridge;	/**< Rock Ridge file or directory name. */
	char *joliet;		/**< Joliet identifier. */
	int jsize;		/**< Size in bytes of the Joliet identifier. */
};

/**
 * POSIX attributes.
 */
struct iso_posix_fields {
	off_t size;		/**< Size of the file or directory. */
	mode_t mode;		/**< File or directory mode. */
	uid_t user;		/**< Owner. */
	gid_t group;		/**< Owning group. */
	time_t mtime;		/**< Date and time of last modification. */
	time_t atime;		/**< Date and time of last access. */
	time_t ctime;		/**< Date and time of last attribute change. */
};

/**
 * Directory on a volume.
 */
struct iso_tree_dir {
	struct iso_tree_dir *parent;		/**< Parent directory. */
	struct iso_volume *volume;		/**< Volume this directory is part of. */

	struct iso_names name;			/**< Directory names. */
	struct iso_posix_fields attrib;		/**< Directory attributes. */
	
	int nchildren;				/**< Number of child directories. */
	int nfiles;				/**< Number of files in this directory. */
	struct iso_tree_dir **children;		/**< Child directories. */
	struct iso_tree_file **files;		/**< Files in this directory. */
};

/**
 * File on a volume.
 */
struct iso_tree_file {
	char *path;				/**< Path of the file. */
	struct iso_tree_dir *parent;		/**< Parent directory. */
	struct iso_volume *volume;		/**< Volume this file is part of. */

	struct iso_names name;			/**< File names. */
	struct iso_posix_fields attrib;		/**< File attributes. */
};

/**
 * Create a new root directory for a volume.
 */
struct iso_tree_dir *iso_tree_new_root(const struct iso_volume *volume);

/**
 * Recursively free a directory.
 */
void iso_tree_free(struct iso_tree_dir *root);

/**
 * Recursively sort all the files and child directories in a directory.
 * Sorting can be considered alphabetical.
 */
void iso_tree_sort(struct iso_tree_dir *root);

#endif /* __ISO_TREE */
