/* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */

#include <ctype.h>
#include <iconv.h>
#include <langinfo.h>
#include <string.h>

#include "util.h"

int valid_d_char(char c)
{
	return (c >= '0' && c <= '9') || (c >= 'A' && c <= 'Z') || (c == '_');
}

int valid_a_char(char c)
{
	return (c >= ' ' && c <= '"') || (c >= '%' && c <= '?') || (c >= 'A' && c <= 'Z') || (c == '_');
}

int valid_j_char(char msb, char lsb)
{
	return !(msb == '\0' && (lsb < ' ' || lsb == '*' || lsb == '/' || lsb == ':' || lsb == ';' || lsb == '?' || lsb == '\\'));    
}

int valid_p_char(char c)
{
	return (c >= '0' && c <= '9') || (c >= 'A' && c <= 'Z') || (c >= 'a' && c <= 'z') || (c == '.') || (c == '_') || (c == '-');
}

char *iso_d_str(const char *src, int size, int pad)
{
	char *dest = strlen(src) > size || pad ? malloc(size + 1) : malloc(strlen(src) + 1);
	int i;
	
	/* Try converting to upper-case before validating. */
	for (i = 0; i < size && src[i]; i++) {
		char c = toupper(src[i]);
		dest[i] = valid_d_char(c) ? c : '_';
	}

	/* Optionally pad with spaces and terminate with NULL. */
	if (pad)
		while (i < size)
			dest[i++] = ' ';
	dest[i] = '\0';

	return dest;
}

char *iso_a_str(const char *src, int size)
{
	char *dest = malloc(size + 1);
	int i;
	
	for (i = 0; i < size && src[i]; i++) {
		char c = toupper(src[i]);
		dest[i] = valid_a_char(c) ? c : '_';
	}
	while (i < size)
		dest[i++] = ' ';
	dest[i] = '\0';

	return dest;
}

char *iso_j_str(const char *src, int size)
{
	char *dest = malloc(size + 2); /* We use size + 2 since we need to add a 16-bit NULL. */
	char *in, *out;
	int inleft, outleft;
	iconv_t cd;

	/* If the conversion is unavailable, return NULL.
	 * Obviously, nl_langinfo(..) requires the locale to be set correctly. */
	cd = iconv_open("UCS-2BE", nl_langinfo(CODESET));
	if (cd == (iconv_t) -1) {
		free(dest);
		return NULL;
	}

	/* In, out, inleft and outleft will be updated by iconv, that's why we need separate variables. */
	in = src;
	out = dest;
	inleft = strlen(src);
	outleft = size;
	iconv(cd, &in, &inleft, &out, &outleft);

	/* Since we need to pad with NULLs, we can pad up to and including the terminating NULLs. */
	outleft += 2;
	while (outleft) {
		*out++ = '\0';
		outleft--;
	}
	iconv_close(cd);

	return dest;
}

char *iso_1_fileid(const char *src)
{
	char *dest = malloc(15); /* 15 = 8 (name) + 1 (.) + 3 (ext) + 2 (;1) + 1 (\0) */
	char *dot = strrchr(src, '.'); /* Position of the last dot in the filename, will be used to calculate lname and lext. */
	int lname, lext, pos, i;

	/* If the dot is the first or last character of the filename, we consider this dot to be part of the filename, else it's a separator. */
	if (dot == NULL || dot == src || *(dot + 1) == '\0') {
		lname = strlen(src);
		lext = 0;
	}
	else {
		lext = strlen(dot + 1);
		lname = strlen(src) - lext - 1;
	}

	/* If we can't build a filename, return NULL. */
	if (lname == 0 && lext == 0) {
		free(dest);
		return NULL;
	}

	pos = 0;
	/* Convert up to 8 characters of the filename. */
	for (i = 0; i < lname && i < 8; i++) {
		char c = toupper(src[i]);
		dest[pos++] = valid_d_char(c) ? c : '_';
	}
	/* This dot is mandatory, even if there is no extension. */
	dest[pos++] = '.';
	/* Convert up to 3 characters of the extension, if any. */
	for (i = 0; i < lext && i < 3; i++) {
		char c = toupper(src[lname + 1 + i]);
		dest[pos++] = valid_d_char(c) ? c : '_';
	}
	/* File versions are mandatory, even if they aren't used. */
	dest[pos++] = ';';
	dest[pos++] = '1';
	dest[pos] = '\0';
	dest = (char *) realloc(dest, pos + 1);

	return dest;
}

char *iso_2_fileid(const char *src)
{
	char *dest = malloc(34); /* 34 = 30 (name + ext) + 1 (.) + 2 (;1) + 1 (\0) */
	char *dot = strrchr(src, '.');
	int lname, lext, lnname, lnext, pos, i;

	/* Since the maximum length can be divided freely over the name and extension, we need to calculate their new lengths (lnname and lnext).
	 * If the original filename is too long, we start by trimming the extension, but keep a minimum extension length of 3. */
	if (dot == NULL || dot == src || *(dot + 1) == '\0') {
		lname = strlen(src);
		lnname = (lname > 30) ? 30 : lname;
		lext = lnext = 0;
	}
	else {
		lext = strlen(dot + 1);
		lname = strlen(src) - lext - 1;
		lnext = (strlen(src) > 31 && lext > 3) ? (lname < 27 ? 30 - lname : 3) : lext;
		lnname = (strlen(src) > 31) ? 30 - lnext : lname;
	}

	if (lnname == 0 && lnext == 0) {
		free(dest);
		return NULL;
	}

	pos = 0;
	/* Convert up to lnname characters of the filename. */
	for (i = 0; i < lnname; i++) {
		char c = toupper(src[i]);
		dest[pos++] = valid_d_char(c) ? c : '_';
	}
	dest[pos++] = '.';
	/* Convert up to lnext characters of the extension, if any. */
	for (i = 0; i < lnext; i++) {
		char c = toupper(src[lname + 1 + i]);
		dest[pos++] = valid_d_char(c) ? c : '_';
	}
	dest[pos++] = ';';
	dest[pos++] = '1';
	dest[pos] = '\0';
	dest = (char *) realloc(dest, pos + 1);

	return dest;
}

char *iso_j_id(const char *src, int *size)
{
	char *dest = malloc(136); /* 136 = 128 (name + ext) + 2 (\0 .) + 4 (\0 ; \0 1) + 2 (\0 \0) */
	char *dot = strrchr(src, '.');
	int lname, lext, lcname, lcext, lnname, lnext, inleft, outleft, pos, i;
	char *cname, *cext, *in, *out;
	iconv_t cd;

	if (dot == NULL || dot == src || *(dot + 1) == '\0') {
		lname = strlen(src);
		lext = 0;
	}
	else {
		lext = strlen(dot + 1);
		lname = strlen(src) - lext - 1;
	}

	if (lname == 0 && lext == 0) {
		free(dest);
		return NULL;
	}

	cd = iconv_open("UCS-2BE", nl_langinfo(CODESET));
	if (cd == (iconv_t) -1) {
		free(dest);
		return NULL;
	}

	/* We need to convert the name and extension first, in order to calculate the number of characters they have. */
	cname = malloc(lname * 2);
	in = src;
	out = cname;
	inleft = lname;
	outleft = lname * 2;
	iconv(cd, &in, &inleft, &out, &outleft);
	lcname = (lname * 2) - outleft;
	iconv_close(cd);

	if (lext) {
		cd = iconv_open("UCS-2BE", nl_langinfo(CODESET));
		if (cd == (iconv_t) -1) {
			free(dest);
			free(cname);
			return NULL;
		}
		cext = malloc(lext * 2);
		in = dot + 1;
		out = cext;
		inleft = lext;
		outleft = lext * 2;
		iconv(cd, &in, &inleft, &out, &outleft);
		lcext = (lext * 2) - outleft;
		iconv_close(cd);
	}
	else
		lcext = 0;

	/* Again, divide the available characters over name and extension, but keep a minimum of three characters for the new extension. */
	lnext = (lcname + lcext > 128 && lcext > 6) ? (lcname < 122 ? 128 - lcname : 6) : lcext;
	lnname = (lcname + lcext > 128) ? 128 - lnext : lcname;

	pos = 0;
	/* Copy up to lnname bytes from the converted filename. */
	for (i = 0; i < lnname; i = i + 2)
		if (valid_j_char(cname[i], cname[i + 1])) {
			dest[pos++] = cname[i];
			dest[pos++] = cname[i + 1];
		}
		else {
			dest[pos++] = '\0';
			dest[pos++] = '_';
		}
	/* Dot is now a 16 bit separator. */
	dest[pos++] = '\0';
	dest[pos++] = '.';
	/* Copy up to lnext bytes from the converted extension, if any. */
	for (i = 0; i < lnext; i = i + 2)
		if (valid_j_char(cext[i], cext[i + 1])) {
			dest[pos++] = cext[i];
			dest[pos++] = cext[i + 1];
		}
		else {
			dest[pos++] = '\0';
			dest[pos++] = '_';
		}
	/* Again, 2 bytes per character. */
	dest[pos++] = '\0';
	dest[pos++] = ';';
	dest[pos++] = '\0';
	dest[pos++] = '1';
	dest[pos++] = '\0';
	dest[pos] = '\0';

	dest = (char *) realloc(dest, pos + 1);
	free(cname);
	free(cext);

	/* Fill in the size in bytes (including the terminating NULLs) of the destination string. */
	if (size != NULL)
		*size = pos + 1;
	return dest;
}

char *iso_p_filename(const char *src)
{
	char *dest = malloc(251); /* We can fit up to 250 characters in a Rock Ridge name entry. */
	char *dot = strrchr(src, '.');
	int lname, lext, lnname, lnext, pos, i;

	if (dot == NULL || dot == src || *(dot + 1) == '\0') {
		lname = strlen(src);
		lnname = (lname > 250) ? 250 : lname;
		lext = lnext = 0;
	}
	else {
		lext = strlen(dot + 1);
		lname = strlen(src) - lext - 1;
		lnext = (strlen(src) > 250 && lext > 3) ? (lname < 246 ? 249 - lname : 3) : lext;
		lnname = (strlen(src) > 250) ? 249 - lnext : lname;
	}

	if (lnname == 0 && lnext == 0) {
		free(dest);
		return NULL;
	}

	pos = 0;
	for (i = 0; i < lnname; i++)
		dest[pos++] = valid_p_char(src[i]) ? src[i] : '_';
	if (lnext) {
		dest[pos++] = '.';
		for (i = 0; i < lnext; i++)
			dest[pos++] = valid_p_char(src[lname + 1 + i]) ? src[lname + 1 + i] : '_';
	}
	dest[pos] = '\0';
	dest = (char *) realloc(dest, pos + 1);

	return dest;
}

char *iso_p_dirname(const char *src)
{
	char *dest = strlen(src) > 250 ? malloc(251) : malloc(strlen(src) + 1);
	int i;
	
	if (strlen(src) == 0) {
		free(dest);
		return NULL;
	}

	for (i = 0; i < 250 && src[i]; i++)
		dest[i] = valid_p_char(src[i]) ? src[i] : '_';
	dest[i] = '\0';

	return dest;
}

char *iso_pathname(const char *dir, const char *file)
{
	char *path = malloc(strlen(dir) + strlen(file) + 2);

	strcpy(path, dir);
	path[strlen(dir)] = '/';
	strcpy(path + strlen(dir) + 1, file);

	return path;
}
