/* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */

/**
 * Utility functions for the Libisofs library.
 */

#ifndef __ISO_UTIL
#define __ISO_UTIL

/**
 * Checks if the given character is a valid d-character.
 */
int valid_d_char(char c);

/**
 * Checks if the given character is a valid a-character.
 */
int valid_a_char(char c);

/**
 * Checks if the given character is part of the Joliet Allowed Character Set.
 */
int valid_j_char(char msb, char lsb);

/**
 * Checks if the given character is part of the POSIX Portable Filename Character Set.
 */
int valid_p_char(char c);

/**
 * Build a string of d-characters of maximum size 'size', optionally padded with spaces if necessary.
 * Can be used to create directory identifiers if padding is turned off.
 */
char *iso_d_str(const char *src, int size, int pad);

/**
 * Build a string of a-characters of maximum size 'size', padded with spaces if necessary.
 */ 
char *iso_a_str(const char *src, int size);

/**
 * Build a string of j-characters of maximum size 'size', padded with NULLs if necessary.
 * Requires the locale to be set correctly.
 * @return NULL if the conversion from the current codeset to UCS-2BE is not available.
 */
char *iso_j_str(const char *src, int size);

/**
 * Create a level 1 file identifier that consists of a name, extension and version number.
 * The resulting string will have a file name of maximum length 8, followed by a separator (.),
 * an optional extension of maximum length 3, followed by a separator (;) and a version number (digit 1).
 * @return NULL if the original name and extension both are of length 0.
 */
char *iso_1_fileid(const char *src);

/**
 * Create a level 2 file identifier that consists of a name, extension and version number.
 * The combined file name and extension length will not exceed 30, the name and extension will be separated (.),
 * and the extension will be followed by a separator (;) and a version number (digit 1).
 * @return NULL if the original name and extension both are of length 0.
 */
char *iso_2_fileid(const char *src);

/**
 * Create a Joliet file or directory identifier that consists of a name, extension and version number.
 * The combined name and extension length will not exceed 128 bytes, the name and extension will be separated (.),
 * and the extension will be followed by a separator (;) and a version number (digit 1).
 * All characters consist of 2 bytes and the resulting string is NULL-terminated by a 2-byte NULL.
 * Requires the locale to be set correctly.
 * @param size will be set to the size (in bytes) of the identifier.
 * @return NULL if the original name and extension both are of length 0 or the conversion from the current codeset to UCS-2BE is not available.
 */
char *iso_j_id(const char *src, int *size);

/**
 * Create a POSIX portable file name that consists of a name and extension.
 * The resulting file name will not exceed 250 characters.
 * @return NULL if the original name and extension both are of length 0.
 */
char *iso_p_filename(const char *src);

/**
 * Create a POSIX portable directory name.
 * The resulting directory name will not exceed 250 characters.
 * @return NULL if the original name is of length 0.
 */
char *iso_p_dirname(const char *src);

/**
 * Build a pathname out of a directory and file name.
 */
char *iso_pathname(const char *dir, const char *file);

#endif /* __ISO_UTIL */
