/* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */

#include <stdlib.h>

#include "libisofs.h"
#include "tree.h"
#include "util.h"
#include "volume.h"

struct iso_volume *iso_volume_new(const char *volume_id, const char *publisher_id, const char *data_preparer_id)
{
	struct iso_volume *volume;

	volume = calloc(1, sizeof(struct iso_volume));
	volume->refcount = 1;

	/* Get a new root directory. */
	volume->root = iso_tree_new_root(volume);

	/* Set these fields, if given. */
	if (volume_id != NULL)
		iso_volume_set_volume_id(volume, volume_id);
	if (publisher_id != NULL)
		iso_volume_set_publisher_id(volume, publisher_id);
	if (data_preparer_id != NULL)
		iso_volume_set_data_preparer_id(volume, data_preparer_id);

	/* Default options. */
	volume->options.iso_level = 2;
	volume->options.rockridge = 1;
	volume->options.joliet = 0;
	
	return volume;
}

void iso_volume_free(struct iso_volume *volume)
{
	/* Only free if no references are in use. */
	if (--volume->refcount < 1) {
		iso_tree_free(volume->root);

		free(volume->volume_id.iso);
		free(volume->volume_id.joliet);
		free(volume->publisher_id.iso);
		free(volume->publisher_id.joliet);
		free(volume->data_preparer_id.iso);
		free(volume->data_preparer_id.joliet);

		free(volume);
	}
}

struct iso_tree_dir *iso_volume_get_root(const struct iso_volume *volume)
{
	return volume->root;
}

void iso_volume_set_volume_id(struct iso_volume *volume, const char *volume_id)
{
	/* This field is 32 bytes long and needs to be padded. */
	volume->volume_id.iso = iso_d_str(volume_id, 32, 1);
	volume->volume_id.joliet = iso_j_str(volume_id, 32);
	volume->volume_id.size = 32;
}

void iso_volume_set_publisher_id(struct iso_volume *volume, const char *publisher_id)
{
	/* This field is 128 bytes long and needs to be padded. */
	volume->publisher_id.iso = iso_a_str(publisher_id, 128);
	volume->publisher_id.joliet = iso_j_str(publisher_id, 128);
	volume->publisher_id.size = 128;
}

void iso_volume_set_data_preparer_id(struct iso_volume *volume, const char *data_preparer_id)
{
	/* This field is 128 bytes long and needs to be padded. */
	volume->data_preparer_id.iso = iso_a_str(data_preparer_id, 128);
	volume->data_preparer_id.joliet = iso_j_str(data_preparer_id, 128);
	volume->data_preparer_id.size = 128;
}

int iso_volume_get_iso_level(const struct iso_volume *volume)
{
	return volume->options.iso_level;
}

void iso_volume_set_iso_level(struct iso_volume *volume, int level)
{
	if (level == 1 || level == 2)
		volume->options.iso_level = level;
}

int iso_volume_get_rockridge(const struct iso_volume *volume)
{
	return volume->options.rockridge;
}

void iso_volume_set_rockridge(struct iso_volume *volume, int rockridge)
{
	volume->options.rockridge = !!rockridge;
}

int iso_volume_get_joliet(const struct iso_volume *volume)
{
	return volume->options.joliet;
}

void iso_volume_set_joliet(struct iso_volume *volume, int joliet)
{
	volume->options.joliet = !!joliet;
}
